﻿unit U_FileExecute;
{
Composant TFileExecute

Développé par:
Matthieu GIROUX
Licence GPL

Composant non visuel permettant d'estraire d'un fichier

}
interface

{$IFDEF FPC}
{$mode Delphi}
{$ENDIF}

uses
  SysUtils, Classes,
{$IFDEF VERSIONS}
   fonctions_version,
{$ENDIF}
   U_ExtAbsCopy,
   process,
   DB;

{$IFDEF VERSIONS}
const
    gVer_FileExecute : T_Version = ( Component : 'Composant TFileExecute' ;
                                               FileUnit : 'U_FileExecute' ;
                                               Owner : 'Matthieu Giroux' ;
                                               Comment : 'Exécution pour des fichiers.' ;
                                               BugsStory :  '0.9.0.0 : In place.';
                                               UnitType : 3 ;
                                               Major : 0 ; Minor : 9 ; Release : 0 ; Build : 0 );

{$ENDIF}

type
    TECopyExeEvent = procedure(Sender : Tobject; const ASource, AExtensionSource, ADestination : String; var as_LineWithoutExe : String ;var AContinue:Boolean) of object;

    { TFileExecute }

    TFileExecute = class(TAbsFileCopy)
     private
       FExe,FNewExtension : String;
       FOnSuccess : TECopyFinishEvent;
       FOnFailure : TECopyErrorEvent;
       FOnCopy    : TECopyExeEvent;
       FOnChange : TEChangeDirectoryEvent;
       FInprogress : Boolean;
       FProcess : TProcess;
     protected
      { Déclarations protégées }
       FInited : Boolean;
       function IsCopyOk(const ai_Error: Integer;  as_Message: String): Boolean; override;
       function  BeforeCopy : Boolean ; override;
       procedure Notification(AComponent: TComponent; Operation: TOperation); override;
     public
      { Déclarations publiques }
       constructor Create ( AOwner : TComponent ) ; override;
       function InternalDefaultCopyFile  ( const as_Source, as_Destination : String ):Boolean; override;
       procedure InternalFinish ( const as_Source, as_Destination : String ); override;
       function CreateDestination ( const as_Destination : String ): Boolean; override;
       procedure PrepareCopy ; virtual;
       property InProgress : Boolean read FInprogress;
     published
       property Exe : String read FExe write FExe;
       property NewExtension : String read FNewExtension write FNewExtension;
       property Process : TProcess read FProcess write FProcess;
       property OnSuccess : TECopyFinishEvent read FOnSuccess write FOnSuccess;
       property OnFailure : TECopyErrorEvent read FOnFailure write FOnFailure;
       property OnCopy    : TECopyExeEvent read FOnCopy write FOnCopy;
       property OnChange : TEChangeDirectoryEvent read FOnChange write FOnChange;
     end;

implementation

uses Forms, TypInfo,
     StrUtils,
{$IFDEF FPC}
     LazFileUtils,
     lazutf8classes,
{$ELSE}
     fonctions_system,
{$ENDIF}
     fonctions_string,
     Dialogs;


{TFileExecute}

// DoInit TFileExecute component
constructor TFileExecute.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FInited := False;
  FInProgress := False;
  FOnChange:=nil;
  FOnFailure:=nil;
  FOnSuccess:=nil;
  FOnCopy:=nil;
  FExe:='';
end;


// verifying Extractd copy
function TFileExecute.IsCopyOk ( const ai_Error : Integer ;  as_Message : String ):Boolean;
begin
  Result := True ;
  if  ( ai_Error <> 0 )
  and assigned ( FOnFailure ) then
     FOnFailure ( Self, ai_Error, as_Message, Result );
End ;

function TFileExecute.BeforeCopy: Boolean;
begin

end;

procedure TFileExecute.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if ( Operation <> opRemove ) Then
   Exit;
  if AComponent = FProcess Then
    Process:=nil;
end;

// Internal finish
procedure TFileExecute.InternalFinish ( const as_Source, as_Destination : String );
begin
  if assigned ( FOnSuccess ) then
      FOnSuccess ( Self, as_Source, as_Destination, 0 );
End ;

function TFileExecute.CreateDestination(const as_Destination: String): Boolean;
begin
  if not DirectoryExistsUTF8(as_Destination)
   Then
       ForceDirectoriesUTF8(as_Destination);
  Result := True;
end;

procedure TFileExecute.PrepareCopy;
begin

end;


function TFileExecute.InternalDefaultCopyFile  ( const as_Source, as_Destination : String ):Boolean;
var as_Line : String;
    LContinue: Boolean;
    ls_destination, ls_Extension : String;
    li_last : Integer;
Begin
  Result := True;
  FInprogress:=True;
  ls_destination:= ExtractFileName(as_Source);
  li_last := fi_LastCharString('.',ls_destination);
  if li_last > 0 Then
    Begin
      ls_Extension:=LowerCase(copy(as_Source,li_last+1,Length(as_Source)));
      ls_destination:=IncludeTrailingPathDelimiter(as_Destination)+copy(ExtractFileName(ls_destination),1,li_last)+FNewExtension;
    end
   Else
   Begin
     ls_Extension:='';
     ls_destination:=IncludeTrailingPathDelimiter(as_Destination)+ExtractFileName(as_Source);
   end;
  as_Line := as_Source+' '+ls_Destination;

  LContinue:=True;
  try
    if FileExistsUTF8(as_Source)
    and Assigned(FOnCopy) Then
     FOnCopy ( Self, as_Source, ls_Extension, ls_Destination , as_Line, LContinue );


    if Assigned(FProcess)
    and LContinue Then
     with FProcess do
      Begin
        Executable:=FExe;
        CommandLine:=FExe+' '+as_Line;
        Parameters.Add(as_Line);
        Execute;
      end;

  finally
    FInprogress:=False;
    InternalFinish(as_Source,as_Destination);
  end;
end;


{$IFDEF VERSIONS}
initialization
  p_ConcatVersion ( gVer_FileExecute );
{$ENDIF}
end.
